#ifndef BACKGROUND_H
#define BACKGROUND_H
#include "gba.h"

///BGCNT defines ///
#define BG_MOSAIC_ENABLE			0x40
#define BG_COLOR_256			0x80
#define BG_COLOR_16				0x0

#define CharBaseBlock(n)		(((n)*0x4000)+0x6000000)
#define ScreenBaseBlock(n)		(((n)*0x800)+0x6000000)

#define SetMosaic(bh,bv,oh,ov) ((bh)+(bv<<4)+(oh<<8)+(ov<<12))

#define BLDMOD_FIRST_BG0  BIT00
#define BLDMOD_FIRST_BG1  BIT01
#define BLDMOD_FIRST_BG2  BIT02
#define BLDMOD_FIRST_BG3  BIT03
#define BLDMOD_FIRST_OBJ  BIT04
#define BLDMOD_FIRST_BD   BIT05
#define BLDMOD_MODE(n)    (n<<6)
#define BLDMOD_SECOND_BG0 BIT08
#define BLDMOD_SECOND_BG1 BIT09
#define BLDMOD_SECOND_BG2 BIT10
#define BLDMOD_SECOND_BG3 BIT11
#define BLDMOD_SECOND_OBJ BIT12
#define BLDMOD_SECOND_BD  BIT13


#define CHAR_SHIFT				2
#define SCREEN_SHIFT			8
#define TEXTBG_SIZE_256x256		0x0
#define TEXTBG_SIZE_256x512		0x8000
#define TEXTBG_SIZE_512x256		0x4000
#define TEXTBG_SIZE_512x512		0xC000

#define ROTBG_SIZE_128x128		0x0
#define ROTBG_SIZE_256x256		0x4000
#define ROTBG_SIZE_512x512		0x8000
#define ROTBG_SIZE_1024x1024	0xC000

#define WRAPAROUND              0x1

#define TILE_FLIP_VERT 0x800
#define TILE_FLIP_HORZ 0x400

typedef struct Bg
{
	u16* tileData;
	u16 * mapData;
	u8 mosiac;
	u8 colorMode;
	u8 number;
	u16 size;
	u8 charBaseBlock;
	u8 screenBaseBlock;
	u8 wraparound;
	s16 x_scroll,y_scroll;
	s32 DX,DY;
	s16 PA,PB,PC,PD;
}Bg;


void EnableBackground(Bg* bg);
void RotateBackground(Bg* bg, unsigned short angle, unsigned short center_x, unsigned short center_y, FIXED zoom);
void UpdateBackground(Bg* bg);
void ClearScreen(Bg * bg);


#endif
