#include "sound.h"

#include "gba.h"
#include"backgrounds.h"
#include "interupts.h"
#include"text16.h"

extern SoundMap * theSoundMap[];

#define SAMPLE_LENGTH(n) ((theSoundMap[n]->pSampleEnd - theSoundMap[n]->pSample)*4)
#define CHANNEL_A 0
#define CHANNEL_B 1 

unsigned long int iNextSample[2]; // index to the next section of sample to play
unsigned long int SampleSize[2]; // the size of the sample
const unsigned long * pSample[2]; // pointer to the current sample playing
unsigned char nLoop[2]; // Loop the sample
unsigned char nLoopSample[2]; // the sample to loop
unsigned char isSoundPlaying[2];

void init_sound(void) {
  
  isSoundPlaying[CHANNEL_A] = 0;
  isSoundPlaying[CHANNEL_B] = 0;
  
}


void TimerPlaySound(const unsigned long int * pS, unsigned long int nSize, int theChannel);

// play a sound effect, the number is the index of the sample defined in the sound_data file
void PlaySoundFX(unsigned char nSample) {
  PlayLoopedSoundFX(nSample, 0);
}

//void debug_stats(void) {
//  int i;
//  char a[4];
//  char str[30];
//  extern Bg bg_text;    
//  for (i = 0; i < 2; i++) {  
//    // debug stats...
//    itoa(isSoundPlaying[i], a, 10);
//    if (i == 0)
//      strcpy(str,  "Sound A ");
//    else
//      strcpy(str,  "Sound B ");
//    strcat(str, a);
//    strcat(str, " ");
//    print_text16(str, 0+i*15, 3, &bg_text,1+i);
//  }
//  
//}


void PlayLoopedSoundFX(unsigned char nSample, unsigned char loops) {
  if (nSample > 20)
   return;
  //REG_IME=0;				//turn off interrupts
  // need to make loop an array of 2, can have 2 samples playing at once then
  // see which channel is free...
  int theChannel = CHANNEL_A;

  if (isSoundPlaying[CHANNEL_A] && !isSoundPlaying[CHANNEL_B]) {
    // play in channel B!
    theChannel = CHANNEL_B;
  }
  
  nLoop[theChannel] = loops;
  nLoopSample[theChannel] = nSample;
  TimerPlaySound(theSoundMap[nSample]->pSample, 
                 SAMPLE_LENGTH(nSample), 
                 theChannel);
//  debug_stats();
  //REG_IME=1;				//enable interrupts
}

/*
--------------------------------------------------------------------------------
Direct Sound source code
By Uze, 2001
http://www.BeLogic.com
You may freely distribute, modify or use this source code without any limitations.

-Created in Visual Studio C++ 6.0
-Compiles with GCC's build "DevKitAdvance"
-Uses Jeff's crt0.s and lnkscript (Crtls v1.26) from http://devrs.com/gba

--------------------------------------------------------------------------------

*/

/*
Address: 0x04000082 - REG_SOUNDCNT_H (Direct Sound control and Sound 1-4 output ratio)
W        W       
F E D C  B A 9 8  7 6 5 4  3 2 1 0
Q P O N  M L K J  X X X X  I H G G
0-1 (G) = Output Sound Ratio for channels 1-4.           
          00 - 25%
          01 - 50%
          10 - 100%
          11 - ?? 
2   (H) = Direct sound A output ratio (0 - 50%, 1 - 100%)
3   (I) = Direct sound B output ratio (0 - 50%, 1 - 100%)
8   (J) = Direct Sound A to right output
9   (K) = Direct sound A to left output
A   (L) = Direct sound A Sampling rate timer (timer 0 or 1). 
          Use this to set which
          timer contorls the playback frequency.
B   (M) = Direct sound A FIFO reset
C   (N) = Direct sound B to right output
D   (O) = Direct sound B to left output
E   (P) = Direct sound B Sampling rate timer (timer 0 or 1). 
          Use this to set which timer controls the playback frequency.
F   (Q) = Direct sound B FIFO reset
This register is used in controlling Direct Sound on the GBA. 
Output ratios control the volume, in percentage, that gets output to the speakers.
*/

// F.I.F.O first in first out


// say which channels are being used...
unsigned char samples[2];

void TimerPlaySound(const unsigned long int * pS, 
                    unsigned long int nSize, 
                    int theChannel) {
	//Play a mono sound at 16khz in Interrupt mode Direct Sound
	//uses timer 0 as sampling rate source
	
  pSample[theChannel] = pS;
  SampleSize[theChannel] = nSize;
  iNextSample[theChannel] = 0;
  isSoundPlaying[theChannel] = 1;
  //DirectSound A + fifo reset + max volume to L and R
  //                  0x800    + 0x100 + 0x200
  
  REG_SOUNDCNT_H |= ENABLE_DS(theChannel);
	//REG_SOUNDCNT_H = DS_A_RATIO_100 | DS_A_RIGHT | DS_A_LEFT | DS_A_TIMER0 | DS_A_FIFO;//ENABLE_A_FULL;
	REG_SOUNDCNT_X=0x0080;  //turn sound chip on

	REG_IE|=INT_TIMER0;		//enable timer 0 irq
	
	//set playback frequency
	//note: using anything else thank clock multipliers to serve as sample frequencies
	//		tends to generate distortion in the output. It has probably to do with timing and 
	//		FIFO reloading.

	REG_TM0CNT_L=0xffff; 
	REG_TM0CNT_H=0x00C3;	//enable timer at CPU freq/1024 +irq =16386Khz sample rate
	
}

void Timer0UpdateSound(void){

	//Interrupt mode Direct Sound IRQ
	//load FIFO each 4 samples with 4 bytes	
	// pSample is a pointer to a unsigned long int
	// i.e. 4 bytes
	
	// check channel A
	if( isSoundPlaying[CHANNEL_A] ) {
    if (!(iNextSample[CHANNEL_A]&3)) {
      REG_SGFIFOA=pSample[CHANNEL_A][iNextSample[CHANNEL_A]>>2];
    }
	  iNextSample[CHANNEL_A]++;  
	}
	if(isSoundPlaying[CHANNEL_B] ) {
	  if( !(iNextSample[CHANNEL_B]&3)) {
	    REG_SGFIFOB=pSample[CHANNEL_B][iNextSample[CHANNEL_B]>>2];
	  }
	  
	  iNextSample[CHANNEL_B]++;
	}
  
  // update the 2 channels
  int i;
  int samples = isSoundPlaying[CHANNEL_A] + isSoundPlaying[CHANNEL_B];
  for (i = 0; i < 2; i ++) {
  	if(iNextSample[i]>SampleSize[i]){
  		//sample finished!
  		if (isSoundPlaying[i])
  		  samples--;
  		isSoundPlaying[i] = 0;
  		// turn sound chip off
  		if (nLoop[i] > 0) 
  		  nLoop[i]--;
  		if (nLoop[i] != 0) {
  		  TimerPlaySound(theSoundMap[nLoopSample[i]]->pSample, 
  		  SAMPLE_LENGTH(nLoopSample[i]),
  		  i);
  		} 
  	}
  }
  if (!samples)
    REG_TM0CNT_H=0;	//disable timer 0
}

