/*****************************\
* 	sound.h
*	By staringmonkey
*	Last modified on 12/04/01
\*****************************/

#ifndef SOUND_H
#define SOUND_H

//////////////////////////Necessary Includes/////////////////////////////
#include "dma.h"
#include "Timers.h"
//////////////////////////////Definitions/////////////////////////////////
//SGCNT0_L
#define RIGHT_VOLUME(n)					n
#define LEFT_VOLUME(n)					n<<4
#define SOUND_1_RIGHT_CHANNEL			BIT08
#define SOUND_2_RIGHT_CHANNEL			BIT09
#define SOUND_3_RIGHT_CHANNEL			BIT10
#define SOUND_4_RIGHT_CHANNEL			BIT11
#define SOUND_1_LEFT_CHANNEL			BIT12
#define SOUND_2_LEFT_CHANNEL			BIT13
#define SOUND_3_LEFT_CHANNEL			BIT14
#define SOUND_4_LEFT_CHANNEL			BIT15

//SGCNT0_H
#define SOUND_OUTPUT_1_4				0
#define SOUND_OUTPUT_1_2				BIT00
#define SOUND_OUTPUT_FULL				BIT01
#define DSOUND_A_OUTPUT_HALF			0
#define DSOUND_A_OUTPUT_FULL			BIT02
#define DSOUND_B_OUTPUT_HALF			0
#define DSOUND_B_OUTPUT_FULL			BIT03
#define DSOUND_A_RIGHT_CHANNEL			BIT08
#define DSOUND_A_LEFT_CHANNEL			BIT09
#define DSOUND_A_TIMER_0				0
#define DSOUND_A_TIMER_1				BIT10
#define DSOUND_A_FIFO_RESET				BIT11
#define DSOUND_B_RIGHT_CHANNEL			BIT12
#define DSOUND_B_LEFT_CHANNEL			BIT13
#define DSOUND_B_TIMER_0				0
#define DSOUND_B_TIMER_1				BIT14
#define DSOUND_B_FIFO_RESET				BIT15

//SGCNT1
#define SOUND_1_ENABLE					BIT00
#define SOUND_2_ENABLE					BIT01
#define SOUND_3_ENABLE					BIT02
#define SOUND_4_ENABLE					BIT03
#define SOUND_MASTER_ENABLE				BIT07

//SG10_L
#define S10L_NUM_SWEEP_SHIFTS(n)		n
#define S10L_SWEEP_INCREASE				0
#define S10L_SWEEP_DECREASE				BIT03
#define S10L_SWEEP_CHANGE_FREQ(n)		n<<4

//SG10_H
#define S10H_SOUND_LENGTH(n)			n
#define S10H_WAVEFORM_CYCLE_12			0
#define S10H_WAVEFORM_CYCLE_25			BIT06
#define S10H_WAVEFORM_CYCLE_50			BIT07
#define S10H_WAVEFROM_CYCLE_75			BIT06 | BIT07
#define S10H_ENVELOPE_STEPS(n)			n<<8
#define S10H_ENVELOPE_INCREASE			0
#define S10H_ENVELOPE_DECREASE			BIT11
#define S10H_ENVELOPE_START_VALUE(n)	n<<12

//SG20
#define S20_SOUND_LENGTH(n)				n
#define S20_WAVEFORM_CYCLE_12			0
#define S20_WAVEFORM_CYCLE_25			BIT06
#define S20_WAVEFORM_CYCLE_50			BIT07
#define S20_WAVEFROM_CYCLE_75			BIT06 | BIT07
#define S20_ENVELOPE_STEPS(n)			n<<8
#define S20_ENVELOPE_INCREASE			0
#define S20_ENVELOPE_DECREASE			BIT11
#define S20_ENVELOPE_START_VALUE(n)		n<<12

//SG21
#define S21_FREQUENCY(n)				n
#define S21_CONTINOUS					0
#define S21_COUNTER						BIT14
#define S21_RESTART						BIT15

//SG30_L
#define S30L_32_STEP_WAVEFORM			0
#define S30L_64_STEP_WAVEFORM			BIT05
#define S30L_RAM_BANK_0					0
#define S30L_RAM_BANK_1					BIT06
#define S30L_ENABLE						BIT07

//SG30_H
#define S30H_SOUND_LENGTH(n)			n
#define S30H_OUTPUT_MUTE				0
#define S30H_OUTPUT_FULL				BIT13
#define S30H_OUTPUT_1_2					BIT14
#define S30H_OUTPUT_1_4					BIT13 | BIT14
#define S30H_OUTPUT_3_4					BIT15

//SG31
#define S31_FREQUENCY(n)				n
#define S31_CONTINOUS					0
#define S31_COUNTER						BIT14
#define S31_RESTART						BIT15

//SG40
#define S40_SOUND_LENGTH(n)				n
#define S40_ENVELOPE_STEPS(n)			n<<8
#define S40_ENVELOPE_INCREASE			0
#define S40_ENVELOPE_DECREASE			BIT11
#define S40_ENVELOPE_START_VALUE(n)		n<<12

//SG41
#define S41_DIVIDE_FREQ_2				0
#define S41_DIVIDE_FREQ_1				BIT00
#define S41_DIVIDE_FREQ_1_2				BIT01
#define S41_DIVIDE_FREQ_1_3				BIT00 | BIT01
#define S41_DIVIDE_FREQ_1_4				BIT02
#define S41_DIVIDE_FREQ_1_5				BIT00 | BIT02
#define S41_DIVIDE_FREQ_1_6				BIT01 | BIT02
#define S41_DIVIDE_FREQ_1_7				BIT00 | BIT01 | BIT02
#define S41_POLY_STEPS_15				0
#define S41_POLY_STEPS_7				BIT03
#define S41_POLY_SHIFT_FREQ(n)			n<<4
#define S41_CONTINOUS					0
#define S41_COUNTER						BIT14
#define S41_RESTART						BIT15


////////////////////////////////Function Prototypes//////////////////////////////
void PlayDirectSoundA(u8 *sound, u16 sampleRate, u32 length, u8 loop);
void PlayDirectSoundB(u8 *sound, u16 sampleRate, u32 length, u8 loop);

void UpdateDirectSoundA(void);
void UpdateDirectSoundB(void);
void StartGBCSound1(u8 leftVolume, u8 rightVolume, u8 channel);
void PlayGBCSound1(u8 leftVolume, u8 rightVolume, u16 length, u16 *sound);
void PlayGBCSound2(u8 leftVolume, u8 rightVolume, u16 length) ;
void UpdateSound1(void);
void UpdateSound2(void);
void DSoundIRQHandler(void);
/////////////////////////////////////Globals/////////////////////////////////////
u8 *soundA;			//Sound A sample
u32 soundALength;	//Sound A sample length
u8 soundALoop;		//Sound A loop parameter

u8 *soundB;			//Sound B sample
u32 soundBLength;	//Sound B sample length
u8 soundBLoop;		//Sound B loop parameter

u16 *sound1;		//Sound 1 sample
u16 sound1Length;	//Sound 1 sample length
u16 sound1Current;	//Sound 1 current sound

u16 *sound2;		//Sound 1 sample
u16 sound2Length;	//Sound 1 sample length
u16 sound2Current;	//Sound 1 current sound

u16 IRQcalls;
u16 IRQremainder;
u16 IRQcalls_made;
////////////////////////////////////PlayDirectSoundA/////////////////////////////////////
void PlayDirectSoundA(u8 *sound, u16 sampleRate, u32 length, u8 loop) 
{
  //Set sound A's current sound
	soundA = sound;
	IRQcalls = 0;
	IRQcalls_made = 0;
	if (length > 0xFFFF) {
	  // requires to call teh IRQ handler multiple times as
	  // it is longer than he length of 1 Timer1 sweep
	  // calc how many times longer, and after that many, repeat or stop the sound
	  IRQcalls = (u16)(length/0xFFFF);
	  IRQremainder = 0xFFFF - (length - (0xFFFF*IRQcalls));
	  REG_TM1CNT_L=0x0;	//0xffff-the number of samples to play
	} else {
	  REG_TM1CNT_L=0xFFFF - length;	//0xffff-the number of samples to play
	  IRQcalls = 0;
	  IRQremainder = 0xFFFF;
	}

	//Set the length and looping
	soundALength = length;
	soundALoop = loop;
  REG_SOUNDCNT_L=0;
	REG_SOUNDCNT_H=0x0b0F;  //DS A&B + fifo reset + timer0 + max volume to L and R
	REG_SOUNDCNT_X=0x0080;  //turn sound chip on
	
	REG_DMA1SAD = (u32)sound;
	REG_DMA1DAD=0x040000a0; //write to FIFO A address
	REG_DMA1CNT_H=0xb600;	//dma control: DMA enabled+ start on FIFO+32bit+repeat
	
	
	REG_TM1CNT_H=0xC4;		//enable timer1 + irq and cascade from timer 0
  REG_INTERUPT = (u32)DSoundIRQHandler;		// Set interrupt proc address (IRQHandler) 
	REG_IE=0x10;	  	//enable irq for timer 1 overflow
	REG_IME=1;				//enable interrupt
	
	//Formula for playback frequency is: 0xFFFF-round(cpuFreq/playbackFreq)
	REG_TM0CNT_L=65536 - (16777216/sampleRate);	//16khz playback freq
	REG_TM0CNT_H=0x0080; 	//enable timer at CPU freq 
	
}

void DSoundIRQHandler(void) {
  REG_IE=0x0;	  	//disable irq 
  if (IRQcalls_made+1 == IRQcalls && IRQcalls != 0) {
    REG_TM1CNT_L=IRQremainder;	// the left over after dividing to get IRQcalls
    // the final call
    IRQcalls_made++;
    REG_IE=0x10;
  } else if (IRQcalls_made == IRQcalls || IRQcalls == 0) {
    // finished!
    if(soundALoop == 1)	{
  		//Play again
  		REG_TM0CNT=0; //disable timer 0
      REG_TM1CNT = 0;
      REG_DMA1CNT_H=0; //stop DMA
      //clear the interrupt(s)
		  PlayDirectSoundA(soundA, 8000, soundALength, 1);
	  }	else	{
  		//End
      //sample finished!,stop Direct sound
      REG_TM0CNT=0; //disable timer 0
      REG_TM1CNT = 0;
      REG_DMA1CNT_H=0; //stop DMA
      //clear the interrupt(s)
      
    }
  } else {
    IRQcalls_made++;
    REG_IE=0x10;
  }
  
  REG_IF |= REG_IF;
}


////////////////////////////////////PlayDirectSoundB/////////////////////////////////////
void PlayDirectSoundB(u8 *sound, u16 sampleRate, u32 length, u8 loop) 
{
	//Output DirectSound A to right channel
	REG_SGCNT0_H = DSOUND_B_RIGHT_CHANNEL | DSOUND_B_LEFT_CHANNEL | DSOUND_B_FIFO_RESET	;

	//Enable all sound
	REG_SGCNT1 = SOUND_MASTER_ENABLE;

	//DMA1 Source Addresss
	REG_DMA1SAD = (u32)sound;

	//Set sound B's current sound
	soundB = sound;

	//Set the length and looping
	soundBLength = length;
	soundBLoop = loop;

	//DMA1 Destination Address (REG_SGFIFOB)
	REG_DMA1DAD = 0x40000A4;

	//Write 32 bits into 0x040000A4 (REG_SGFIF0B) every VSync
	REG_DMA1CNT_H =  DMA_DEST_FIXED_8BIT | DMA_REPEAT_8BIT | DMA_32_8BIT | DMA_TIMING_SYNC_TO_DISPLAY_8BIT | DMA_ENABLE_8BIT;

	//Sample Rate
	REG_TM0D = 65536 - (16777216/sampleRate);

	//Enable the timer
	REG_TM0CNT = TIME_ENABLE;
}

/////////////////////////////////////UpdateDirectSoundA////////////////////////////////
void UpdateDirectSoundA(void)
{
	//Check current
	if(REG_DMA1SAD >= ((u32)soundA + soundALength))
	{
		if(soundALoop == 1)
		{
			//Play again
			PlayDirectSoundA(soundA, 8000, soundALength, 1);
		}
		else
		{
			//End
			REG_TM0CNT = 0;			
		}
	}
}

/////////////////////////////////////UpdateDirectSoundB////////////////////////////////
void UpdateDirectSoundB(void)
{
	//Check current
	if(REG_DMA1SAD >= ((u32)soundB + soundBLength))
	{
		if(soundBLoop == 1)
		{
			//Play again
			PlayDirectSoundB(soundB, 8000, soundBLength, 1);
		}
		else
		{
			//End
			REG_TM0CNT = 0;			
		}
	}
}

void StartGBCSound1(u8 leftVolume, u8 rightVolume, u8 channel) {
  u32 channels_on;
  switch (channel) {
    case 1: channels_on = SOUND_1_LEFT_CHANNEL | SOUND_1_RIGHT_CHANNEL ; break;
    case 2: channels_on = SOUND_2_LEFT_CHANNEL | SOUND_2_RIGHT_CHANNEL ; break;
    case 3: channels_on = SOUND_3_LEFT_CHANNEL | SOUND_3_RIGHT_CHANNEL ; break;
    case 4: channels_on = SOUND_4_LEFT_CHANNEL | SOUND_4_RIGHT_CHANNEL ; break;
  }
  
  REG_SGCNT0_L = LEFT_VOLUME(leftVolume) | RIGHT_VOLUME(rightVolume) | channels_on;
	REG_SGCNT0_H = SOUND_OUTPUT_FULL;
	//Enable all sound
  REG_SGCNT1 = SOUND_MASTER_ENABLE;
}

////////////////////////////////////PlayGBCSound1/////////////////////////////////////
void PlayGBCSound1(u8 leftVolume, u8 rightVolume, u16 length, u16 *sound) 
{
	//Set volumes and 
	REG_SGCNT0_L = LEFT_VOLUME(leftVolume) | RIGHT_VOLUME(rightVolume) | SOUND_1_RIGHT_CHANNEL | SOUND_1_LEFT_CHANNEL;

	//Set sound output
	REG_SGCNT0_H = SOUND_OUTPUT_FULL;

	//Enable all sound
	REG_SGCNT1 = SOUND_MASTER_ENABLE;

	//Set the sweep change frequency to 1
	REG_SG10_L = S10L_SWEEP_CHANGE_FREQ(1);

	//Sound length to 24, envelope start at 15
	REG_SG10_H = S10H_SOUND_LENGTH(length) | S10H_ENVELOPE_START_VALUE(15);
  sound1 = sound;
	//Write the first sample, set the length, and 0 out the currentSound counter
	REG_SG11 = 32768 + sound1[0];
	sound1Length = length;
	sound1Current = 0;
}

////////////////////////////////////PlayGBCSound2/////////////////////////////////////
void PlayGBCSound2(u8 leftVolume, u8 rightVolume, u16 length) 
{
	//Set volumes and 
	REG_SGCNT0_L = LEFT_VOLUME(leftVolume) | RIGHT_VOLUME(rightVolume) | SOUND_2_RIGHT_CHANNEL | SOUND_2_LEFT_CHANNEL;

	//Set sound output
	REG_SGCNT0_H = SOUND_OUTPUT_FULL;

	//Enable all sound
	REG_SGCNT1 = SOUND_MASTER_ENABLE;

	//Sound length to 24, envelope start at 15
	REG_SG20 = S20_SOUND_LENGTH(24) | S20_ENVELOPE_START_VALUE(15);

	//Write the first sample, set the length, and 0 out the currentSound counter
	REG_SG21 = 32768 + sound2[0];
	sound2Length = length;
	sound2Current = 0;
}

/////////////////////////////////////UpdateSound1////////////////////////////////
void UpdateSound1(void)
{
	//Cycle through the sounds
	if(sound1Current + 1 < sound1Length)
	{
		sound1Current++;
	}
	else
	{
		sound1Current = 0;
	}

	//Set current sound
	REG_SG11 = sound1[sound1Current];
}

/////////////////////////////////////UpdateSound2////////////////////////////////
void UpdateSound2(void)
{
	//Cycle through the sounds
	if(sound2Current + 1 < sound2Length)
	{
		sound2Current++;
	}
	else
	{
		sound2Current = 0;
	}

	//Set current sound
	REG_SG21 = sound2[sound2Current];
}



#endif
