#include "gba.h"
#include "screenmode.h"
#include "text16.h"
#include "dma.h"

u16 bgmapoffset16; 
void reverse(char s[]);

// set up the 16 point letters in the tile memory - used for standard stuff...
// the letters are 4bpp
void setup_text16(Bg* bg, u16 offset) {
  bgmapoffset16 = offset;
  u16 realos = bgmapoffset16*16; // each 8 by 8 tile takes up 16 bytes
  
  // the font consists of 84 letters
  // each one is 16*8, 4bpp = 16*4 pixels = 64 u8 transfers, 32 u16 transfers per char
  // total = 32*84
  REG_DMA3SAD = (u32)&text16_raw[0]; 
  REG_DMA3DAD = (u32)&bg->tileData[realos];
  REG_DMA3CNT = 32*MAX_LETTERS |DMA_16NOW;
  
  // the colour is set depending on the palette
  
}

// set the colour for the text in a given palette
void set_text16_colour(u8 pal, u16 col) {
  BGPaletteMem[16*pal+1] = col;
}

unsigned short get_text16_colour(u8 pal) {
  return BGPaletteMem[16*pal+1];
}



// print the 16 point text
// each letter is 2 tiles big. the first tile goes on row y,
// the second on row y+1
void print_text16(char* text, u16 x, u16 y , Bg* bg, u8 pal)
{
	u32 indx=0;
	u16 newx = x;
	
	u16 letter;
  while( (text[indx]!= 0) ) 
  {
    letter = text[indx] - FIRST_CHAR_INDEX;
    // draw a letter...
    if (text[indx] != 32) {
      bg->mapData[newx+y*32] = letter*2+bgmapoffset16;
      bg->mapData[newx+(y+1)*32] = letter*2 + 1 + bgmapoffset16;
    }
    else {
      letter = TRANSP_CHAR_INDEX - FIRST_CHAR_INDEX;
      bg->mapData[newx+y*32] = letter*2+1+bgmapoffset16;//9+bgmapoffset16+97;
      bg->mapData[newx+(y+1)*32] = letter*2+1+bgmapoffset16;//9+bgmapoffset16+97;
    }
    // set the palette at these locations too
    bg->mapData[newx+y*32] &= ~0xF000; 
    bg->mapData[newx+y*32] |= (pal<<12)&0xF000;
    
    bg->mapData[newx+(y+1)*32] &= ~0xF000; 
    bg->mapData[newx+(y+1)*32] |= (pal<<12)&0xF000;
    
    newx++;
    indx++;
  }
}


void clear_bg(Bg* bg)  {
  u8 x, y;
  for (x = 0; x < 32; x++) {
    for (y = 0; y < 32; y++) {
      bg->mapData[x+y*32] = 0; 
    }
  }
}


/*
 *  remove whitespace at the end and the beginning of a char array
 *  the char array is used for the name, so max length is 11
 */
 
char * trim_whitespace(char * str) {
  char *s = str;
  char *d = str;
  
  // eat trailing white space
  while (*s)
    s++;
  while (str < s && isspace(*(s-1)))
    *s-- = '\0';
  
  *s = '\0';
  
  s = str;
  // has leading space?
  if (isspace(*s))
  {
    // eat white space 
    while (isspace(*s))
      s++;
    
    // copy
    while (*s)
      *d++ = *s++;
    *d = '\0';
  } 
  return str;
}

void itoa(int n, char s[], int b) {
    static char digits[16] = "0123456789ABCDEF";
    int i, sign;
   
    if ( b < 2 || b > 36 ) {
        return;
    }
   
    if ((sign = n) < 0)
        n = -n;
    i = 0;
    do {
        s[i++] = digits[n % b];
    } while ((n /= b) > 0);
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
}


/*  Reverses string s[] in place  */

void reverse(char s[]) {
    int c, i, j;
    for ( i = 0, j = strlen(s)-1; i < j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

