#include "gba.h"
#include "screenmode.h"
#include "text8.h"
#include "dma.h"

u16 bgmapoffset8; 

// set up the 8 point letters in the tile memory - used for standard stuff...
// the letters are 4bpp
void setup_text8(Bg* bg, u16 offset) {
  bgmapoffset8 = offset;
  u16 realos = bgmapoffset8*16; // each 8 by 8 tile takes up 16 bytes
  
  // the font consists of 256 characters
  // each one is 8*8, 4bpp = 8*4 pixels = 32 u8 transfers, 16 u16 transfers per char
  // total = 16*256
  unsigned long i;
  unsigned long index = 0;
  for (i = 0; i < 16*MAX_LETTERS; i++) {
    bg->tileData[realos+i] = (text8_raw[index]|(text8_raw[index+1]<<8));
    index+=2;
  }
  
//  REG_DMA3SAD = (u32)&text8_raw[4]; 
//  REG_DMA3DAD = (u32)&bg->tileData[realos];
//  REG_DMA3CNT = 16*MAX_LETTERS |DMA_16NOW;
  
  // the colour is set depending on the palette
  
}

// set the colour for the text in a given palette
void set_text8_colour(u8 pal, u16 col1, u16 col2) {
  BGPaletteMem[16*pal+1] = col1;
  BGPaletteMem[16*pal+5] = col1;
  BGPaletteMem[16*pal+2] = col2;
}

void get_text8_colour(u8 pal, u16 * col1, u16 *col2) {
  *col1 = BGPaletteMem[16*pal+1];
  *col2 = BGPaletteMem[16*pal+2];
  
}



// print the 16 point text
// each letter is 1 tile big. 
void print_text8(char* text, u16 x, u16 y , Bg* bg, u8 pal)
{
	u32 indx=0;
	u16 newx = x;
	
	u16 letter;
  while( (text[indx]!= 0) ) 
  {
    letter = text[indx] - FIRST_CHAR_INDEX;
    // draw a letter...
    if (text[indx] != 32) {
      bg->mapData[newx+y*32] = letter+bgmapoffset8;
    }
    else {
      letter = TRANSP_CHAR_INDEX - FIRST_CHAR_INDEX;
      bg->mapData[newx+y*32] = letter+bgmapoffset8;
    }
    // set the palette at these locations too
    bg->mapData[newx+y*32] &= ~0xF000; 
    bg->mapData[newx+y*32] |= (pal<<12)&0xF000;
    
    newx++;
    indx++;
  }
}


void clear_bg(Bg* bg)  {
  u8 x, y;
  for (x = 0; x < 32; x++) {
    for (y = 0; y < 32; y++) {
      bg->mapData[x+y*32] = 0; 
    }
  }
}


/*
 *  remove whitespace at the end and the beginning of a char array
 */
char * trim_whitespace(char * str) {
  char *s = str;
  char *d = str;
  
  // eat trailing white space
  while (*s)
    s++;
  while (str < s && isspace(*(s-1)))
    *s-- = '\0';
  
  *s = '\0';
  
  s = str;
  // has leading space?
  if (isspace(*s))
  {
    // eat white space 
    while (isspace(*s))
      s++;
    
    // copy
    while (*s)
      *d++ = *s++;
    *d = '\0';
  } 
  return str;
}

