#include "text_funcs.h"
#include "letters_bg_gfx.h"
#include "dma.h"
#include "UnpackLZ77.h"
#include "gba.h"
#include "screenmode.h"


void itob(int n, char s[], int b);
void reverse(char s[]);
void ScreenToMem(u16 *nX);

u16 bgmapoffset;
u8 * pLetters;

// a hacky way to set up the letters in the tile memory - used for standard stuff...
void SetUpLetters(Bg* bg, u16 offset) {
  bgmapoffset = offset;
  u16 realos = bgmapoffset*8*4;
  
  UnpackLetters();
  
  // transfer in the Letters
  REG_DMA3SAD = (u32)&pLetters[0]; 
  REG_DMA3DAD = (u32)&bg->tileData[realos];
  REG_DMA3CNT = 3104 |DMA_16NOW;
  
  // set default colours
  BGPaletteMem[TEXT_COL] = RGB16(31,31,31);
//  BGPaletteMem[BG_EDGE_COL] = bgpalette[0x0f];
//  BGPaletteMem[BG_CENTER_COL] = bgpalette[0x0A];
  
}

// unpack the letters but do not copy them to the bg
void UnpackLetters(void) {
  if (pLetters != 0) {
    free(pLetters);
    pLetters = 0;
  }
  u16 nSize;
  LZ77GetUnpackedSize(letters_raw, &nSize);
  pLetters = (u8*) calloc(nSize, sizeof(u8));
  LZ77UnCompWRAM(letters_raw,pLetters);

}


void SetUpLetters2(Bg* bg, u16 offset, u8 nCol) {
  // nCol is a colour fpr the background, replace all the 0x00 with nCol...
  bgmapoffset = offset;
  u16 realos = bgmapoffset*8*4;
  
  // transfer in the Letters
  u32 i; // for counter
  u32 index = 0;
  for (i = 0; i < 3104*2; i +=2) {
    u8 lo = pLetters[i];
    u8 hi = pLetters[i+1];
    if (lo == 0x00) 
      lo = nCol;
    if (hi == 0x00) 
      hi = nCol;
    bg->tileData[realos+index] = lo|hi<<8;
    index++;
  }
  
  // set default colours
  BGPaletteMem[TEXT_COL] = RGB16(31,31,31);
//  BGPaletteMem[BG_EDGE_COL] = bgpalette[0x0f];
//  BGPaletteMem[BG_CENTER_COL] = bgpalette[0x0A];
  
}

void ScreenToMem(u16 *nX) { 
  u16 factor = (*nX)>>2;
  u16 remainder = (*nX) - factor * 4;
  u32 newX = factor*32 + remainder;
  *nX = newX;
  
//  u16 factor = (*nX)>>3;
//  u16 remainder = (*nX) - factor * 8;
//  u32 newX = factor*64 + remainder;
//  *nX = newX;
  
}

u32 WriteString(Bg* bg, const char * cTxt, u16 nX, u8 nY) {
  
  u16 x_offset = nX;  
  u16 newY = nY;
  // need to do this word at a time...
  u16 i = 0;
  u8 x,y;
  while (cTxt[i] > 0) {
    u16 j = i;
    u16 width = 0;
    u16 tmp_nLargestX = 0;
    
    if (cTxt[i] != 32 && cTxt[i] != '\n') {
      while (cTxt[j] != 0 && cTxt[j] != 32 && cTxt[j] != '\n') {
        u32 letter_offset = (cTxt[j]-0x21)*64;
        tmp_nLargestX = 0;
        for (y=0; y<8; y++) {
          for (x=0; x<4; x++) {
  
            if (pLetters[letter_offset+x*2+y*8] != 0 && x*2 > tmp_nLargestX) {
             // we have a new large X position..
              tmp_nLargestX = x*2;
            }
            if ( (pLetters[letter_offset+x*2+1+y*8]) != 0 && x*2+1 > tmp_nLargestX) {
              tmp_nLargestX = x*2+1;
            }
          } 
        }
        tmp_nLargestX++;
        u8 nLXdiv2 = (tmp_nLargestX>>1);
        // add half the largestX value to the x_offset, rounding up where needed..
        width += ((nLXdiv2<<1)==tmp_nLargestX)?nLXdiv2:1+nLXdiv2;
//        width += tmp_nLargestX;
        j++;
      }

      u16 tmp_newX = x_offset+width;
      ScreenToMem(&tmp_newX);


      if (tmp_newX > 955) { // not all the way, because it glitches with spaces..
        // new line - return the text pointer... 
        // write to the end of the line from x_offset
        tmp_newX = x_offset;
        ScreenToMem(&tmp_newX);
        // count how many spaces are needed...
        u16 nSpaces  = (960 - tmp_newX)/4; // space left, divided by 4
//        nSpaces += 1; // extra just in case and the 0
        char *cSpaces = (char*)malloc((nSpaces+1)*sizeof(char));
        if (cSpaces==0) 
          while(1){}
        u8 nLoop;
        for (nLoop = 0; nLoop < nSpaces; nLoop++) {
          cSpaces[nLoop] = NON_SEP_BLANK; // non seperated blank space unicode
        }
        cSpaces[nSpaces] = '\0';
        WriteWord(bg, cSpaces, x_offset, newY);
        free(cSpaces);
        
        
        /*
        while (spaces > 0) {
          tmp_newX = x_offset;
          ScreenToMem(&tmp_newX);
          if (tmp_newX > 960) {
            return i;
          }
          WriteWord(bg, " ", x_offset, newY);
          if (spaces > 3)
            spaces -=4;
          else {
            WriteWord(bg, " ", x_offset+4, newY);
            spaces = 0;
          }
          x_offset += 4;
        }
        */
        return i;
      }
      char *txt = (char*)malloc(1+(j-i)*sizeof(char));
      txt [j-i] = '\0';
      u16 k;
      for (k = i; k < j; k++) {
        txt[k-i] = cTxt[k];
      }
      WriteWord(bg, txt, x_offset, newY);
      free(txt);
      x_offset += width;
      
      i+=(j-i);
    } else if (cTxt[i] == 32) {
      u16 tmp_newX = x_offset+4;
      ScreenToMem(&tmp_newX);
      if (tmp_newX > 960) {        
        return i+1;
      }
      WriteWord(bg, " ", x_offset, newY);
      x_offset += 4;
      i++;
    } else if (cTxt[i] == '\n') {
      u16 tmp_newX = x_offset;
      ScreenToMem(&tmp_newX);
      u16 spaces  = 960-tmp_newX;
      while (spaces > 4) {
        tmp_newX = x_offset+4;
        ScreenToMem(&tmp_newX);
        if (tmp_newX > 960) {
          i++;
          return i;
        }
        WriteWord(bg, " ", x_offset, newY);
        x_offset += 4;
        spaces -=4;
      }
      i++;
      return i;
    } else {
      i++;
    }
  }
  // reached the end of the text, clear to the end of the line
  u16 tmp_newX = x_offset;
  ScreenToMem(&tmp_newX);
  u16 spaces  = 960-tmp_newX;
  while (spaces > 4) {
    tmp_newX = x_offset+4;
    ScreenToMem(&tmp_newX);
    if (tmp_newX > 960) {
      return i;
    }
    WriteWord(bg, " ", x_offset, newY);
    x_offset += 4;
    spaces -=4;
  }
  return i;
}


void WriteWord(Bg* bg, const char * cTxt, u16 nX, u8 nY) {
  // writes a word to the screen at the given nX position of row nY
  // writes a string to the x position of row y ... assumes tiles are set up going 0-600
//  qprint(0, "WORD!", 0, 1,bg);
  /*
  can draw text "bunched up"
  inputs:
      Bg * bg the background to draw to
      cTxt - text to draw
      nX -> x pos 
      nY -> row 
  */

  u16 ln = 0;
  while (cTxt[ln] > 0) {
    ln++;
  }
  
  u8 x, y;
  u16 i;
  u32 nLargestX = 0;
  u16 newX = nX;
  u16 newY = nY;
  ScreenToMem(&newX);
  u16 max_x = 960;
  if (newX > max_x) {
    newX = newX - max_x;
    newY++;
  }
  u32 offset = 0;//newY*30*8*4;
  u32 x_offset = nX;
  offset = newX + newY*30*8*4;
//  qprint(newX, cTxt, 0, 3, bg);
//  WaitForVsync();
//  WriteWordGen(cTxt, &bg->tileData[offset], 0xDE);
  
  for (i = 0; i < ln; i++) {
    // need to check for the next word here, and if it can't be added onto this line, new line
    
    if (cTxt[i] != 32 && cTxt[i] != '\n') { // not space
      u32 letter_offset = (cTxt[i]-0x21)*64;
//      x_offset += nLargestX;
      u8 nLXdiv2 = (nLargestX>>1);
      // add half the largestX value to the x_offset, rounding up where needed..
      x_offset += ((nLXdiv2<<1)==nLargestX)?nLXdiv2:1+nLXdiv2;
      nLargestX = 0;
      for (y=0; y<8; y++) {
        for (x=0; x<4; x++) {
          newX = x+x_offset;
          
          ScreenToMem(&newX);
//          WaitForVsync();
          if (newX > max_x) {
            x_offset = newX - max_x;
            newY+=1;
          }
          offset = newX + newY*30*8*4;
          bg->tileData[offset+y*4] = (pLetters[letter_offset+x*2+y*8])|
          (pLetters[letter_offset+x*2+1+y*8]<<8);
          
          if (pLetters[letter_offset+x*2+y*8] != 0 && x*2 > nLargestX) {
           // we have a new large X position..
            nLargestX = x*2;
          }
          if ( (pLetters[letter_offset+x*2+1+y*8]) != 0 && x*2+1 > nLargestX) {
            nLargestX = x*2+1;
          }
        } 
      }
      nLargestX ++;
    } else if (cTxt[i] == 32) {
      u8 nLXdiv2 = (nLargestX>>1);
      // add half the largestX value to the x_offset, rounding up where needed..
      x_offset += ((nLXdiv2<<1)==nLargestX)?nLXdiv2:1+nLXdiv2;
      nLargestX = 0;
      for (y=0; y<8; y++) {
        for (x=0; x<4; x++) {
          newX = x+x_offset;
          ScreenToMem(&newX);
          if (newX > max_x) {
            x_offset = newX - max_x;
            newY +=1;
          }
          offset = newX + newY*30*8*4;
          bg->tileData[offset+y*4] = 0x0000;
        } 
      }
      nLargestX = 8;
//      qprint(i, "space!", 0, 3, bg);
    } else if (cTxt[i] == '\n') {
      // new line...
//      qprint(0, "NOO!", 0, 1,bg);
      x_offset = 0;
      newY += 1;
      nLargestX = 0;
    }
  }
//  */
}

u32 WriteWordGen(const char * cTxt, u16 *nMemLoc, u8 nCol) {
  // writes a word to the screen at the given nX position of row nY
  // writes a string to the x position of row y ... assumes tiles are set up going 0-600
  // returns the length of the string in pixels
//  qprint(0, "WORD!", 0, 1,bg);
  /*
  can draw text "bunched up"
  inputs:
      Bg * bg the background to draw to
      cTxt - text to draw
      nX -> x pos 
      nY -> row 
  */
  
  u16 ln = 0;
  while (cTxt[ln] > 0) {
    ln++;
  }
  
  u8 x, y;
  u16 i;
  u32 nLargestX = 0;
  u16 newX = 0;
  u16 newY = 0;
//  ScreenToMem(&newX);
  u16 max_x = 4096;
//  if (newX > max_x) {
//    newX = newX - max_x;
//    newY++;
//  }
  u32 offset = 0;//newY*30*8*4;
  u32 x_offset = 0;
  for (i = 0; i < ln; i++) {
    // need to check for the next word here, and if it can't be added onto this line, new line
    
    if (cTxt[i] != 32 && cTxt[i] != '\n') { // not space
      u32 letter_offset = (cTxt[i]-0x21)*64;
      u8 nLXdiv2 = (nLargestX>>1);
      // add half the largestX value to the x_offset, rounding up where needed..
      x_offset += ((nLXdiv2<<1)==nLargestX)?nLXdiv2:1+nLXdiv2;
//      if ((x_offset >> 1)*2 != x_offset)
//        x_offset++;
      nLargestX = 0;
      for (y=0; y<8; y++) {
        for (x=0; x<4; x++) {
          newX = x+x_offset;
          
          ScreenToMem(&newX);
//          WaitForVsync();
          if (newX > max_x) {
            x_offset = newX - max_x;
            newY+=1;
          }
          offset = newX + newY*960; //30*8*4;
          u8 nLo = pLetters[letter_offset+x*2+y*8];
          u8 nHi = pLetters[letter_offset+x*2+1+y*8];
          if (nLo == TEXT_COL)
            nLo = nCol;
          if (nHi == TEXT_COL)
            nHi = nCol;
          nMemLoc[offset+y*4] = (nLo)|(nHi<<8);
          
          if (nLo > 0 && x*2 > nLargestX) {
           // we have a new large X position..
            nLargestX = x*2;
          }
          if ( nHi > 0 && x*2+1 > nLargestX) {
            nLargestX = x*2+1;
          }
        } 
      }
      nLargestX++;
    } else if (cTxt[i] == 32) {
      u8 nLXdiv2 = (nLargestX>>1);
      // add half the largestX value to the x_offset, rounding up where needed..
      x_offset += ((nLXdiv2<<1)==nLargestX)?nLXdiv2:1+nLXdiv2;
      
      nLargestX = 0;
      for (y=0; y<8; y++) {
        for (x=0; x<4; x++) {
          newX = x+x_offset;
          ScreenToMem(&newX);
          if (newX > max_x) {
            x_offset = newX - max_x;
            newY +=1;
          }
          offset = newX + newY*960; //30*8*4;
          nMemLoc[offset+y*4] = 0x0000;
        } 
      }
      nLargestX = 8;
//      qprint(i, "space!", 0, 3, bg);
    } else if (cTxt[i] == '\n') {
      // new line...
//      qprint(0, "NOO!", 0, 1,bg);
      x_offset = 0;
      newY += 1;
      nLargestX = 0;
    }
  }
  return x_offset*2;
}

// debug helper, puts a value and a comment on the screen, less than 10 wide!
void qprint(u32 val, const char* text, u8 x, u8 y, Bg * bg) {
  
  
  
//  WriteWord(bg,"            ", x*8, y);
  qtext("      ", x, y, bg);
  char val_txt[8];
  val_txt[0] = '\0';
  itob(val, val_txt, 10);
  char msg[16];
  msg[0]='\0';
  strcat(msg, text);
  strcat(msg, val_txt);
  
//  WriteWord(bg,msg, x*8, y);
  qtext(msg, x, y, bg);
}


void qtext(char* text, u16 x, u16 y , Bg* bg)
{
	u32 indx=0;
	u16 newx = x;
	
	u16 letter;
   	while( (text[indx]!= 0) ) 
   	{
   	  letter = text[indx]-0x21; // first thing is !, index 0x21
   	  if (text[indx] != 32)
   	    bg->mapData[newx+y*32] = letter+bgmapoffset;
   	  else 
   	    bg->mapData[newx+y*32] = 0;//9+bgmapoffset+97;
   	  newx++;
   	  indx++;
	}
}

void itob(int n, char s[], int b) {
    static char digits[16] = "0123456789ABCDEF";
    int i, sign;
   
    if ( b < 2 || b > 36 ) {
        return;
    }
   
    if ((sign = n) < 0)
        n = -n;
    i = 0;
    do {
        s[i++] = digits[n % b];
    } while ((n /= b) > 0);
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
}


/*  Reverses string s[] in place  */

void reverse(char s[]) {
    int c, i, j;
    for ( i = 0, j = strlen(s)-1; i < j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

